@@ -19,11 +19,13 @@ SITES = {
     'douyu'            : 'douyutv',
     'ehow'             : 'ehow',
     'facebook'         : 'facebook',
+    'fantasy'          : 'fantasy',
     'fc2'              : 'fc2video',
     'flickr'           : 'flickr',
     'freesound'        : 'freesound',
     'fun'              : 'funshion',
     'google'           : 'google',
+    'giphy'            : 'giphy',
     'heavy-music'      : 'heavymusic',
     'huaban'           : 'huaban',
     'huomao'           : 'huomaotv',
@@ -61,6 +63,7 @@ SITES = {
     'pinterest'        : 'pinterest',
     'pixnet'           : 'pixnet',
     'pptv'             : 'pptv',
+    'qingting'         : 'qingting',
     'qq'               : 'qq',
     'quanmin'          : 'quanmin',
     'showroom-live'    : 'showroom',
@@ -92,17 +95,16 @@ SITES = {
     'miaopai'          : 'yixia',
     'yizhibo'          : 'yizhibo',
     'youku'            : 'youku',
+    'iwara'            : 'iwara',
     'youtu'            : 'youtube',
     'youtube'          : 'youtube',
     'zhanqi'           : 'zhanqi',
 }
 
-import getopt
 import json
 import locale
 import logging
 import os
-import platform
 import re
 import socket
 import sys
@@ -110,6 +112,7 @@ import time
 from urllib import request, parse, error
 from http import cookiejar
 from importlib import import_module
+import argparse
 
 from .version import __version__
 from .util import log, term
@@ -161,6 +164,20 @@ def rc4(key, data):
         out_list.append(char ^ prn)
 
     return bytes(out_list)
+
+def general_m3u8_extractor(url, headers={}):
+    m3u8_list = get_content(url, headers=headers).split('\n')
+    urls = []
+    for line in m3u8_list:
+        line = line.strip()
+        if line and not line.startswith('#'):
+            if line.startswith('http'):
+                urls.append(line)
+            else:
+                seg_url = parse.urljoin(url, line)
+                urls.append(seg_url)
+    return urls
+
 def maybe_print(*s):
     try: print(*s)
     except: pass
@@ -325,11 +342,14 @@ def get_location(url):
     return response.geturl()
 
 def urlopen_with_retry(*args, **kwargs):
-    for i in range(10):
+    for i in range(2):
         try:
             return request.urlopen(*args, **kwargs)
         except socket.timeout:
             logging.debug('request attempt %s timeout' % str(i + 1))
+# try to tackle youku CDN fails
+        except error.HTTPError as http_error:
+            logging.debug('HTTP Error with code{}'.format(http_error.code))
 
 def get_content(url, headers={}, decoded=True):
     """Gets the content of a URL via sending a HTTP GET request.
@@ -459,6 +479,9 @@ def url_info(url, faker = False, headers = {}):
         'video/x-ms-asf': 'asf',
         'audio/mp4': 'mp4',
         'audio/mpeg': 'mp3',
+        'audio/wav': 'wav',
+        'audio/x-wav': 'wav',
+        'audio/wave': 'wav',
         'image/jpeg': 'jpg',
         'image/png': 'png',
         'image/gif': 'gif',
@@ -502,11 +525,13 @@ def url_locations(urls, faker = False, headers = {}):
         locations.append(response.url)
     return locations
 
-def url_save(url, filepath, bar, refer = None, is_part = False, faker = False, headers = {}, timeout = None, **kwargs):
-#When a referer specified with param refer, the key must be 'Referer' for the hack here
+
+def url_save(url, filepath, bar, refer=None, is_part=False, faker=False, headers=None, timeout=None, **kwargs):
+    tmp_headers = headers.copy() if headers is not None else {}
+# When a referer specified with param refer, the key must be 'Referer' for the hack here
     if refer is not None:
-        headers['Referer'] = refer
-    file_size = url_size(url, faker = faker, headers = headers)
+        tmp_headers['Referer'] = refer
+    file_size = url_size(url, faker=faker, headers=tmp_headers)
 
     if os.path.exists(filepath):
         if not force and file_size == os.path.getsize(filepath):
@@ -540,20 +565,23 @@ def url_save(url, filepath, bar, refer = None, is_part = False, faker = False, h
 
     if received < file_size:
         if faker:
-            headers = fake_headers
+            tmp_headers = fake_headers
+        '''
+        if parameter headers passed in, we have it copied as tmp_header
         elif headers:
             headers = headers
         else:
             headers = {}
+        '''
         if received:
-            headers['Range'] = 'bytes=' + str(received) + '-'
+            tmp_headers['Range'] = 'bytes=' + str(received) + '-'
         if refer:
-            headers['Referer'] = refer
+            tmp_headers['Referer'] = refer
 
         if timeout:
-            response = urlopen_with_retry(request.Request(url, headers=headers), timeout=timeout)
+            response = urlopen_with_retry(request.Request(url, headers=tmp_headers), timeout=timeout)
         else:
-            response = urlopen_with_retry(request.Request(url, headers=headers))
+            response = urlopen_with_retry(request.Request(url, headers=tmp_headers))
         try:
             range_start = int(response.headers['content-range'][6:].split('/')[0].split('-')[0])
             end_length = int(response.headers['content-range'][6:].split('/')[1])
@@ -570,13 +598,18 @@ def url_save(url, filepath, bar, refer = None, is_part = False, faker = False, h
 
         with open(temp_filepath, open_mode) as output:
             while True:
-                buffer = response.read(1024 * 256)
+                buffer = None
+                try:
+                    buffer = response.read(1024 * 256)
+                except socket.timeout:
+                    pass
                 if not buffer:
                     if received == file_size: # Download finished
                         break
-                    else: # Unexpected termination. Retry request
-                        headers['Range'] = 'bytes=' + str(received) + '-'
-                        response = urlopen_with_retry(request.Request(url, headers=headers))
+                    # Unexpected termination. Retry request
+                    tmp_headers['Range'] = 'bytes=' + str(received) + '-'
+                    response = urlopen_with_retry(request.Request(url, headers=tmp_headers))
+                    continue
                 output.write(buffer)
                 received += len(buffer)
                 if bar:
@@ -588,76 +621,6 @@ def url_save(url, filepath, bar, refer = None, is_part = False, faker = False, h
         os.remove(filepath) # on Windows rename could fail if destination filepath exists
     os.rename(temp_filepath, filepath)
 
-def url_save_chunked(url, filepath, bar, dyn_callback=None, chunk_size=0, ignore_range=False, refer=None, is_part=False, faker=False, headers={}):
-    def dyn_update_url(received):
-        if callable(dyn_callback):
-            logging.debug('Calling callback %s for new URL from %s' % (dyn_callback.__name__, received))
-            return dyn_callback(received)
-    if os.path.exists(filepath):
-        if not force:
-            if not is_part:
-                if bar:
-                    bar.done()
-                print('Skipping %s: file already exists' % tr(os.path.basename(filepath)))
-            else:
-                if bar:
-                    bar.update_received(os.path.getsize(filepath))
-            return
-        else:
-            if not is_part:
-                if bar:
-                    bar.done()
-                print('Overwriting %s' % tr(os.path.basename(filepath)), '...')
-    elif not os.path.exists(os.path.dirname(filepath)):
-        os.mkdir(os.path.dirname(filepath))
-
-    temp_filepath = filepath + '.download'
-    received = 0
-    if not force:
-        open_mode = 'ab'
-
-        if os.path.exists(temp_filepath):
-            received += os.path.getsize(temp_filepath)
-            if bar:
-                bar.update_received(os.path.getsize(temp_filepath))
-    else:
-        open_mode = 'wb'
-
-    if faker:
-        headers = fake_headers
-    elif headers:
-        headers = headers
-    else:
-        headers = {}
-    if received:
-        url = dyn_update_url(received)
-        if not ignore_range:
-            headers['Range'] = 'bytes=' + str(received) + '-'
-    if refer:
-        headers['Referer'] = refer
-
-    response = urlopen_with_retry(request.Request(url, headers=headers))
-
-    with open(temp_filepath, open_mode) as output:
-        this_chunk = received
-        while True:
-            buffer = response.read(1024 * 256)
-            if not buffer:
-                break
-            output.write(buffer)
-            received += len(buffer)
-            if chunk_size and (received - this_chunk) >= chunk_size:
-                url = dyn_callback(received)
-                this_chunk = received
-                response = urlopen_with_retry(request.Request(url, headers=headers))
-            if bar:
-                bar.update_received(len(buffer))
-
-    assert received == os.path.getsize(temp_filepath), '%s == %s == %s' % (received, os.path.getsize(temp_filepath))
-
-    if os.access(filepath, os.W_OK):
-        os.remove(filepath) # on Windows rename could fail if destination filepath exists
-    os.rename(temp_filepath, filepath)
 
 class SimpleProgressBar:
     term_size = term.get_terminal_size()[1]
@@ -901,84 +864,6 @@ def download_urls(urls, title, ext, total_size, output_dir='.', refer=None, merg
 
     print()
 
-def download_urls_chunked(urls, title, ext, total_size, output_dir='.', refer=None, merge=True, faker=False, headers = {}, **kwargs):
-    assert urls
-    if dry_run:
-        print('Real URLs:\n%s\n' % urls)
-        return
-
-    if player:
-        launch_player(player, urls)
-        return
-
-    title = tr(get_filename(title))
-
-    filename = '%s.%s' % (title, ext)
-    filepath = os.path.join(output_dir, filename)
-    if total_size:
-        if not force and os.path.exists(filepath[:-3] + '.mkv'):
-            print('Skipping %s: file already exists' % filepath[:-3] + '.mkv')
-            print()
-            return
-        bar = SimpleProgressBar(total_size, len(urls))
-    else:
-        bar = PiecesProgressBar(total_size, len(urls))
-
-    if len(urls) == 1:
-        parts = []
-        url = urls[0]
-        print('Downloading %s ...' % tr(filename))
-        filepath = os.path.join(output_dir, filename)
-        parts.append(filepath)
-        url_save_chunked(url, filepath, bar, refer = refer, faker = faker, headers = headers, **kwargs)
-        bar.done()
-
-        if not merge:
-            print()
-            return
-        if ext == 'ts':
-            from .processor.ffmpeg import has_ffmpeg_installed
-            if has_ffmpeg_installed():
-                from .processor.ffmpeg import ffmpeg_convert_ts_to_mkv
-                if ffmpeg_convert_ts_to_mkv(parts, os.path.join(output_dir, title + '.mkv')):
-                    for part in parts:
-                        os.remove(part)
-                else:
-                    os.remove(os.path.join(output_dir, title + '.mkv'))
-            else:
-                print('No ffmpeg is found. Conversion aborted.')
-        else:
-            print("Can't convert %s files" % ext)
-    else:
-        parts = []
-        print('Downloading %s.%s ...' % (tr(title), ext))
-        for i, url in enumerate(urls):
-            filename = '%s[%02d].%s' % (title, i, ext)
-            filepath = os.path.join(output_dir, filename)
-            parts.append(filepath)
-            #print 'Downloading %s [%s/%s]...' % (tr(filename), i + 1, len(urls))
-            bar.update_piece(i + 1)
-            url_save_chunked(url, filepath, bar, refer = refer, is_part = True, faker = faker, headers = headers)
-        bar.done()
-
-        if not merge:
-            print()
-            return
-        if ext == 'ts':
-            from .processor.ffmpeg import has_ffmpeg_installed
-            if has_ffmpeg_installed():
-                from .processor.ffmpeg import ffmpeg_concat_ts_to_mkv
-                if ffmpeg_concat_ts_to_mkv(parts, os.path.join(output_dir, title + '.mkv')):
-                    for part in parts:
-                        os.remove(part)
-                else:
-                    os.remove(os.path.join(output_dir, title + '.mkv'))
-            else:
-                print('No ffmpeg is found. Merging aborted.')
-        else:
-            print("Can't merge %s files" % ext)
-
-    print()
 
 def download_rtmp_url(url,title, ext,params={}, total_size=0, output_dir='.', refer=None, merge=True, faker=False):
     assert url
@@ -1027,7 +912,7 @@ def playlist_not_supported(name):
         raise NotImplementedError('Playlist is not supported for ' + name)
     return f
 
-def print_info(site_info, title, type, size):
+def print_info(site_info, title, type, size, **kwargs):
     if json_output:
         json_output_.print_info(site_info=site_info, title=title, type=type, size=size)
         return
@@ -1085,6 +970,8 @@ def print_info(site_info, title, type, size):
         type_info = "MPEG-4 audio (%s)" % type
     elif type in ['audio/mpeg']:
         type_info = "MP3 (%s)" % type
+    elif type in ['audio/wav', 'audio/wave', 'audio/x-wav']:
+        type_info = 'Waveform Audio File Format ({})'.format(type)
 
     elif type in ['image/jpeg']:
         type_info = "JPEG Image (%s)" % type
@@ -1092,14 +979,22 @@ def print_info(site_info, title, type, size):
         type_info = "Portable Network Graphics (%s)" % type
     elif type in ['image/gif']:
         type_info = "Graphics Interchange Format (%s)" % type
-
+    elif type in ['m3u8']:
+        if 'm3u8_type' in kwargs:
+            if kwargs['m3u8_type'] == 'master':
+                type_info = 'M3U8 Master {}'.format(type)
+        else:
+            type_info = 'M3U8 Playlist {}'.format(type)
     else:
         type_info = "Unknown type (%s)" % type
 
     maybe_print("Site:      ", site_info)
     maybe_print("Title:     ", unescape_html(tr(title)))
     print("Type:      ", type_info)
-    print("Size:      ", round(size / 1048576, 2), "MiB (" + str(size) + " Bytes)")
+    if type != 'm3u8':
+        print("Size:      ", round(size / 1048576, 2), "MiB (" + str(size) + " Bytes)")
+    if type == 'm3u8' and 'm3u8_url' in kwargs:
+        print('M3U8 Url:   {}'.format(kwargs['m3u8_url']))
     print()
 
 def mime_to_container(mime):
@@ -1167,12 +1062,9 @@ def print_more_compatible(*args, **kwargs):
     return ret
 
 
-
 def download_main(download, download_playlist, urls, playlist, **kwargs):
     for url in urls:
-        if url.startswith('https://'):
-            url = url[8:]
-        if not url.startswith('http://'):
+        if re.match(r'https?://', url) is None:
             url = 'http://' + url
 
         if playlist:
@@ -1180,216 +1072,211 @@ def download_main(download, download_playlist, urls, playlist, **kwargs):
         else:
             download(url, **kwargs)
 
-def script_main(script_name, download, download_playlist, **kwargs):
-    def version():
-        log.i('version %s, a tiny downloader that scrapes the web.'
-              % get_version(kwargs['repo_path']
-            if 'repo_path' in kwargs else __version__))
+def load_cookies(cookiefile):
+    global cookies
+    try:
+        cookies = cookiejar.MozillaCookieJar(cookiefile)
+        cookies.load()
+    except Exception:
+        import sqlite3
+        cookies = cookiejar.MozillaCookieJar()
+        con = sqlite3.connect(cookiefile)
+        cur = con.cursor()
+        try:
+            cur.execute("""SELECT host, path, isSecure, expiry, name, value
+                        FROM moz_cookies""")
+            for item in cur.fetchall():
+                c = cookiejar.Cookie(
+                    0, item[4], item[5], None, False, item[0],
+                    item[0].startswith('.'), item[0].startswith('.'),
+                    item[1], False, item[2], item[3], item[3]=="", None,
+                    None, {},
+                )
+                cookies.set_cookie(c)
+        except Exception:
+            pass
+        # TODO: Chromium Cookies
+        # SELECT host_key, path, secure, expires_utc, name, encrypted_value
+        # FROM cookies
+        # http://n8henrie.com/2013/11/use-chromes-cookies-for-easier-downloading-with-python-requests/
 
+def set_socks_proxy(proxy):
+    try:
+        import socks
+        socks_proxy_addrs = proxy.split(':')
+        socks.set_default_proxy(socks.SOCKS5,
+                                socks_proxy_addrs[0],
+                                int(socks_proxy_addrs[1]))
+        socket.socket = socks.socksocket
+        def getaddrinfo(*args):
+            return [(socket.AF_INET, socket.SOCK_STREAM, 6, '', (args[0], args[1]))]
+        socket.getaddrinfo = getaddrinfo
+    except ImportError:
+        log.w('Error importing PySocks library, socks proxy ignored.'
+            'In order to use use socks proxy, please install PySocks.')
+
+def script_main(download, download_playlist, **kwargs):
     logging.basicConfig(format='[%(levelname)s] %(message)s')
 
-    help = 'Usage: %s [OPTION]... [URL]...\n\n' % script_name
-    help += '''Startup options:
-    -V | --version                      Print version and exit.
-    -h | --help                         Print help and exit.
-    \n'''
-    help += '''Dry-run options: (no actual downloading)
-    -i | --info                         Print extracted information.
-    -u | --url                          Print extracted information with URLs.
-         --json                         Print extracted URLs in JSON format.
-    \n'''
-    help += '''Download options:
-    -n | --no-merge                     Do not merge video parts.
-         --no-caption                   Do not download captions.
-                                        (subtitles, lyrics, danmaku, ...)
-    -f | --force                        Force overwriting existed files.
-    -F | --format <STREAM_ID>           Set video format to STREAM_ID.
-    -O | --output-filename <FILE>       Set output filename.
-    -o | --output-dir <PATH>            Set output directory.
-    -p | --player <PLAYER [OPTIONS]>    Stream extracted URL to a PLAYER.
-    -c | --cookies <COOKIES_FILE>       Load cookies.txt or cookies.sqlite.
-    -x | --http-proxy <HOST:PORT>       Use an HTTP proxy for downloading.
-    -y | --extractor-proxy <HOST:PORT>  Use an HTTP proxy for extracting only.
-         --no-proxy                     Never use a proxy.
-    -s | --socks-proxy <HOST:PORT>      Use an SOCKS5 proxy for downloading.
-    -t | --timeout <SECONDS>            Set socket timeout.
-    -d | --debug                        Show traceback and other debug info.
-    -I | --input-file                   Read non-playlist urls from file.
-    -P | --password <PASSWORD>          Set video visit password to PASSWORD.
-    '''
+    def print_version():
+        log.i('version %s, a tiny downloader that scrapes the web.'
+                % get_version(kwargs['repo_path']
+            if 'repo_path' in kwargs else __version__))
 
-    short_opts = 'Vhfiuc:ndF:O:o:p:x:y:s:t:I:P:'
-    opts = ['version', 'help', 'force', 'info', 'url', 'cookies', 'no-caption', 'no-merge', 'no-proxy', 'debug', 'json', 'format=', 'stream=', 'itag=', 'output-filename=', 'output-dir=', 'player=', 'http-proxy=', 'socks-proxy=', 'extractor-proxy=', 'lang=', 'timeout=', 'input-file=', 'password=']
-#dead code? download_playlist is a function and always True
-#if download_playlist:
-    short_opts = 'l' + short_opts
-    opts = ['playlist'] + opts
+    parser = argparse.ArgumentParser(
+        prog='you-get',
+        usage='you-get [OPTION]... URL...',
+        description='A tiny downloader that scrapes the web',
+        add_help=False,
+    )
+    parser.add_argument('-V', '--version', action='store_true',
+                        help='Print version and exit')
+    parser.add_argument('-h', '--help', action='store_true',
+                        help='Print this help message and exit')
 
-    try:
-        opts, args = getopt.gnu_getopt(sys.argv[1:], short_opts, opts)
-    except getopt.GetoptError as err:
-        log.e(err)
-        log.e("try 'you-get --help' for more options")
-        sys.exit(2)
+    dry_run_grp = parser.add_argument_group('Dry-run options', '(no actual downloading)')
+    dry_run_grp = dry_run_grp.add_mutually_exclusive_group()
+    dry_run_grp.add_argument('-i', '--info', action='store_true',
+                        help='Print extracted information')
+    dry_run_grp.add_argument('-u', '--url', action='store_true',
+                        help='Print extracted information with URLs')
+    dry_run_grp.add_argument('--json', action='store_true',
+                        help='Print extracted URLs in JSON format')
+
+    download_grp = parser.add_argument_group('Download options')
+    download_grp.add_argument('-n', '--no-merge', action='store_true', default=False,
+                        help='Do not merge video parts')
+    download_grp.add_argument('--no-caption', action='store_true',
+                        help='Do not download captions (subtitles, lyrics, danmaku, ...)')
+    download_grp.add_argument('-f', '--force', action='store_true', default=False,
+                        help='Force overwriting existing files')
+    download_grp.add_argument('-F', '--format', metavar='STREAM_ID',
+                        help='Set video format to STREAM_ID')
+    download_grp.add_argument('-O', '--output-filename', metavar='FILE',
+                        help='Set output filename')
+    download_grp.add_argument('-o', '--output-dir', metavar='DIR', default='.',
+                        help='Set output directory')
+    download_grp.add_argument('-p', '--player', metavar='PLAYER',
+                        help='Stream extracted URL to a PLAYER')
+    download_grp.add_argument('-c', '--cookies', metavar='COOKIES_FILE',
+                        help='Load cookies.txt or cookies.sqlite')
+    download_grp.add_argument('-t', '--timeout', metavar='SECONDS', type=int, default=600,
+                        help='Set socket timeout')
+    download_grp.add_argument('-d', '--debug', action='store_true',
+                        help='Show traceback and other debug info')
+    download_grp.add_argument('-I', '--input-file', metavar='FILE', type=argparse.FileType('r'),
+                        help='Read non-playlist URLs from FILE')
+    download_grp.add_argument('-P', '--password',
+                        help='Set video visit password to PASSWORD')
+    download_grp.add_argument('-l', '--playlist', action='store_true',
+                        help='Prefer to download a playlist')
+
+    proxy_grp = parser.add_argument_group('Proxy options')
+    proxy_grp = proxy_grp.add_mutually_exclusive_group()
+    proxy_grp.add_argument('-x', '--http-proxy', metavar='HOST:PORT',
+                        help='Use an HTTP proxy for downloading')
+    proxy_grp.add_argument('-y', '--extractor-proxy', metavar='HOST:PORT',
+                        help='Use an HTTP proxy for extracting only')
+    proxy_grp.add_argument('--no-proxy', action='store_true',
+                        help='Never use a proxy')
+    proxy_grp.add_argument('-s', '--socks-proxy', metavar='HOST:PORT',
+                        help='Use an SOCKS5 proxy for downloading')
+
+    download_grp.add_argument('--stream',
+                        help=argparse.SUPPRESS)
+    download_grp.add_argument('--itag',
+                        help=argparse.SUPPRESS)
+
+    parser.add_argument('URL', nargs='*',
+                        help=argparse.SUPPRESS)
+
+    args = parser.parse_args()
+
+    if args.help:
+        print_version()
+        parser.print_help()
+        sys.exit()
+    if args.version:
+        print_version()
+        sys.exit()
+
+    if args.debug:
+        # Set level of root logger to DEBUG
+        logging.getLogger().setLevel(logging.DEBUG)
 
     global force
     global dry_run
     global json_output
     global player
     global extractor_proxy
-    global cookies
     global output_filename
 
-    info_only = False
-    playlist = False
+    output_filename = args.output_filename
+    extractor_proxy = args.extractor_proxy
+
+    info_only = args.info
+    if args.url:
+        dry_run = True
+    if args.json:
+        json_output = True
+        # to fix extractors not use VideoExtractor
+        dry_run = True
+        info_only = False
+
+    if args.cookies:
+        load_cookies(args.cookies)
+
     caption = True
-    merge = True
-    stream_id = None
-    lang = None
-    output_dir = '.'
-    proxy = None
-    socks_proxy = None
-    extractor_proxy = None
-    traceback = False
-    timeout = 600
-    urls_from_file = []
-    password = None
+    stream_id = args.format or args.stream or args.itag
+    if args.no_caption:
+        caption = False
+    if args.player:
+        player = args.player
+        caption = False
 
-    for o, a in opts:
-        if o in ('-V', '--version'):
-            version()
-            sys.exit()
-        elif o in ('-h', '--help'):
-            version()
-            print(help)
-            sys.exit()
-        elif o in ('-f', '--force'):
-            force = True
-        elif o in ('-i', '--info'):
-            info_only = True
-        elif o in ('-u', '--url'):
-            dry_run = True
-        elif o in ('--json', ):
-            json_output = True
-            # to fix extractors not use VideoExtractor
-            dry_run = True
-            info_only = False
-        elif o in ('-c', '--cookies'):
-            try:
-                cookies = cookiejar.MozillaCookieJar(a)
-                cookies.load()
-            except:
-                import sqlite3
-                cookies = cookiejar.MozillaCookieJar()
-                con = sqlite3.connect(a)
-                cur = con.cursor()
-                try:
-                    cur.execute("SELECT host, path, isSecure, expiry, name, value FROM moz_cookies")
-                    for item in cur.fetchall():
-                        c = cookiejar.Cookie(0, item[4], item[5],
-                                             None, False,
-                                             item[0],
-                                             item[0].startswith('.'),
-                                             item[0].startswith('.'),
-                                             item[1], False,
-                                             item[2],
-                                             item[3], item[3]=="",
-                                             None, None, {})
-                        cookies.set_cookie(c)
-                except: pass
-                # TODO: Chromium Cookies
-                # SELECT host_key, path, secure, expires_utc, name, encrypted_value FROM cookies
-                # http://n8henrie.com/2013/11/use-chromes-cookies-for-easier-downloading-with-python-requests/
-
-        elif o in ('-l', '--playlist'):
-            playlist = True
-        elif o in ('--no-caption',):
-            caption = False
-        elif o in ('-n', '--no-merge'):
-            merge = False
-        elif o in ('--no-proxy',):
-            proxy = ''
-        elif o in ('-d', '--debug'):
-            traceback = True
-            # Set level of root logger to DEBUG
-            logging.getLogger().setLevel(logging.DEBUG)
-        elif o in ('-F', '--format', '--stream', '--itag'):
-            stream_id = a
-        elif o in ('-O', '--output-filename'):
-            output_filename = a
-        elif o in ('-o', '--output-dir'):
-            output_dir = a
-        elif o in ('-p', '--player'):
-            player = a
-            caption = False
-        elif o in ('-x', '--http-proxy'):
-            proxy = a
-        elif o in ('-s', '--socks-proxy'):
-            socks_proxy = a
-        elif o in ('-y', '--extractor-proxy'):
-            extractor_proxy = a
-        elif o in ('--lang',):
-            lang = a
-        elif o in ('-t', '--timeout'):
-            timeout = int(a)
-        elif o in ('-P', '--password',):
-            password = a
-        elif o in ('-I', '--input-file'):
-            logging.debug('you are trying to load urls from {}'.format(a))
-            if playlist:
-                log.e("reading playlist from a file is unsupported and won't make your life easier")
-                sys.exit(2)
-            with open(a, 'r') as input_file:
-                for line in input_file:
-                    url = line.strip()
-                    urls_from_file.append(url)
-        else:
-            log.e("try 'you-get --help' for more options")
-            sys.exit(2)
-    if not args and not urls_from_file:
-        print(help)
-        sys.exit()
-    args.extend(urls_from_file)
-
-    if (socks_proxy):
-        try:
-            import socket
-            import socks
-            socks_proxy_addrs = socks_proxy.split(':')
-            socks.set_default_proxy(socks.SOCKS5,
-                                    socks_proxy_addrs[0],
-                                    int(socks_proxy_addrs[1]))
-            socket.socket = socks.socksocket
-            def getaddrinfo(*args):
-                return [(socket.AF_INET, socket.SOCK_STREAM, 6, '', (args[0], args[1]))]
-            socket.getaddrinfo = getaddrinfo
-        except ImportError:
-            log.w('Error importing PySocks library, socks proxy ignored.'
-                'In order to use use socks proxy, please install PySocks.')
+    if args.no_proxy:
+        set_http_proxy('')
     else:
-        import socket
-        set_http_proxy(proxy)
+        set_http_proxy(args.http_proxy)
+    if args.socks_proxy:
+        set_socks_proxy(args.socks_proxy)
 
-    socket.setdefaulttimeout(timeout)
+    URLs = []
+    if args.input_file:
+        logging.debug('you are trying to load urls from %s', args.input_file)
+        if args.playlist:
+            log.e("reading playlist from a file is unsupported and won't make your life easier")
+            sys.exit(2)
+        URLs.extend(args.input_file.read().splitlines())
+        args.input_file.close()
+    URLs.extend(args.URL)
+
+    if not URLs:
+        parser.print_help()
+        sys.exit()
+
+    socket.setdefaulttimeout(args.timeout)
 
     try:
+        extra = {}
+        if extractor_proxy:
+            extra['extractor_proxy'] = extractor_proxy
         if stream_id:
-            if not extractor_proxy:
-                download_main(download, download_playlist, args, playlist, stream_id=stream_id, output_dir=output_dir, merge=merge, info_only=info_only, json_output=json_output, caption=caption)
-            else:
-                download_main(download, download_playlist, args, playlist, stream_id=stream_id, extractor_proxy=extractor_proxy, output_dir=output_dir, merge=merge, info_only=info_only, json_output=json_output, caption=caption)
-        else:
-            if not extractor_proxy:
-                download_main(download, download_playlist, args, playlist, output_dir=output_dir, merge=merge, info_only=info_only, json_output=json_output, caption=caption)
-            else:
-                download_main(download, download_playlist, args, playlist, extractor_proxy=extractor_proxy, output_dir=output_dir, merge=merge, info_only=info_only, json_output=json_output, caption=caption)
+            extra['stream_id'] = stream_id
+        download_main(
+            download, download_playlist,
+            URLs, args.playlist,
+            output_dir=args.output_dir, merge=not args.no_merge,
+            info_only=info_only, json_output=json_output, caption=caption, password=args.password,
+            **extra
+        )
     except KeyboardInterrupt:
-        if traceback:
+        if args.debug:
             raise
         else:
             sys.exit(1)
     except UnicodeEncodeError:
-        if traceback:
+        if args.debug:
             raise
         log.e('[error] oops, the current environment does not seem to support Unicode.')
         log.e('please set it to a UTF-8-aware locale first,')
@@ -1399,7 +1286,7 @@ def script_main(script_name, download, download_playlist, **kwargs):
         log.e('    (Linux)      $ LC_CTYPE=en_US.UTF-8')
         sys.exit(1)
     except Exception:
-        if not traceback:
+        if not args.debug:
             log.e('[error] oops, something went wrong.')
             log.e('don\'t panic, c\'est la vie. please try the following steps:')
             log.e('  (1) Rule out any network problem.')
@@ -1410,7 +1297,7 @@ def script_main(script_name, download, download_playlist, **kwargs):
             log.e('  (4) Run the command with \'--debug\' option,')
             log.e('      and report this issue with the full output.')
         else:
-            version()
+            print_version()
             log.i(args)
             raise
         sys.exit(1)
@@ -1436,7 +1323,7 @@ def url_to_module(url):
         video_host = r1(r'https?://([^/]+)/', url)
         video_url = r1(r'https?://[^/]+(.*)', url)
         assert video_host and video_url
-    except:
+    except AssertionError:
         url = google_search(url)
         video_host = r1(r'https?://([^/]+)/', url)
         video_url = r1(r'https?://[^/]+(.*)', url)
@@ -1451,7 +1338,11 @@ def url_to_module(url):
         return import_module('.'.join(['you_get', 'extractors', SITES[k]])), url
     else:
         import http.client
-        conn = http.client.HTTPConnection(video_host)
+        video_host = r1(r'https?://([^/]+)/', url) # .cn could be removed
+        if url.startswith('https://'):
+            conn = http.client.HTTPSConnection(video_host)
+        else:
+            conn = http.client.HTTPConnection(video_host)
         conn.request("HEAD", video_url, headers=fake_headers)
         res = conn.getresponse()
         location = res.getheader('location')
@@ -1469,4 +1360,4 @@ def any_download_playlist(url, **kwargs):
     m.download_playlist(url, **kwargs)
 
 def main(**kwargs):
-    script_main('you-get', any_download, any_download_playlist, **kwargs)
+    script_main(any_download, any_download_playlist, **kwargs)
